# -*- coding: utf8 -*-
# Copyright (c) 2021 Nicholas de Jong

__title__ = "elasticsearch_kibana_cli"
__author__ = 'Nicholas de Jong <contact@nicholasdejong.com>'
__version__ = '0.3.7'
__license__ = 'BSD2'

__cli_name__ = 'eskbcli'
__env_config_filename__ = 'ESKBCLI_CONFIG_FILENAME'
__search_split_count_default__ = 5
__search_split_bucket_limit__ = 10000
__search_default_timeout_seconds__ = 60
